"""
MCP (Model Context Protocol) server for djust framework.

Provides AI assistants with structured access to djust project introspection,
system checks, and code generation tools.

Usage:
    python manage.py djust_mcp        # Start stdio MCP server
    python -m djust.mcp               # Start without Django (framework schema only)
"""
